﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.MultiObjectiveTestFunction;

namespace HeuristicLab.Problems.TestFunctions {
  [Item("DTLZ5", " http://repository.ias.ac.in/81671/ [30.11.15]")]
  [StorableClass]
  public class DTLZ5 : MultiObjectiveTestFunction {

    private int actualSolutionSize = 2;
    public override int ActualSolutionSize {
      get {
        return actualSolutionSize;
      }

      set {
        actualSolutionSize = value;
      }
    }

    public override DoubleMatrix Bounds {
      get {
        return new DoubleMatrix(new double[,] { { 0, 1 } });
      }
    }

    public override bool[] Maximization {
      get {
        return new bool[actualSolutionSize];
      }
    }

    public override int MaximumProblemSize {
      get {
        return int.MaxValue;
      }
    }

    public override int MaximumSolutionSize {
      get {

        return int.MaxValue;
      }
    }

    public override int MinimumProblemSize {
      get {
        return Math.Max(2, ActualSolutionSize+1);
      }
    }

    public override int MinimumSolutionSize {
      get {
        return 2;
      }
    }

    [StorableConstructor]
    protected DTLZ5(bool deserializing) : base(deserializing) { }
    protected DTLZ5(DTLZ5 original, Cloner cloner) : base(original, cloner) { }
    public DTLZ5() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new DTLZ5(this, cloner);
    }



    public override double[] Evaluate(RealVector r) {
      return Evaluate(r, ActualSolutionSize);
    }

    private double[] Evaluate(RealVector r, int objectives) {
      if (r.Length < objectives) {
        throw new Exception("The dimensionality of the problem(ProblemSize) must be larger than the dimensionality of the solution(SolutionSize) ");
      }
      double[] res = new double[objectives];

      //calculate g(Xm)
      double g = 0;
      for (int i = objectives; i < r.Length; i++) {
        g += Math.Pow(r[i], 0.1);
      }

      //phi definition
      Func<double, double> phi;
      phi = (double x) => { return Math.PI / (4 * 1 + g) * (1 + 2 * g * x); };

      //calculating f0...fM-1
      for (int i = 0; i < objectives; i++) {
        double f = i == 0 ? 1 : (Math.Sin(phi(r[objectives - i - 1]) * Math.PI / 2)) * (1 + g);
        for (int j = 0; j < objectives - i - 1; j++) {
          f *= Math.Cos(phi(r[j]) * Math.PI / 2);
        }
        res[i] = f;
      }
      return res;
    }
  }
}
